@@ -104,6 +104,7 @@ static word *sdl_stick_button_state = NULL;
 static word sdl_sticks_joybits = 0;
 static int sdl_mouse_grabbed = 0;
 static unsigned int scancodes[SDLK_LAST];
+extern boolean sdl_fullscreen;
 #endif
 
 
@@ -213,7 +214,7 @@ static int sdl_mouse_motion_filter(SDL_Event const *event)
     } /* if */
     else
     {
-        if (sdl_mouse_grabbed)
+        if (sdl_mouse_grabbed || sdl_fullscreen)
         {
           	mouse_relative_x = event->motion.xrel;
        	    mouse_relative_y = event->motion.yrel;
@@ -459,11 +460,14 @@ static int sdl_key_filter(const SDL_Event *event)
          (event->key.state == SDL_PRESSED) &&
          (event->key.keysym.mod & KMOD_CTRL) )
     {
+      if (!sdl_fullscreen)
+      {
         sdl_mouse_grabbed = ((sdl_mouse_grabbed) ? 0 : 1);
         if (sdl_mouse_grabbed)
             grab_mode = SDL_GRAB_ON;
         SDL_WM_GrabInput(grab_mode);
-        return(0);
+      }
+      return(0);
     } /* if */
 
     else if ( ( (event->key.keysym.sym == SDLK_RETURN) ||
@@ -471,15 +475,19 @@ static int sdl_key_filter(const SDL_Event *event)
               (event->key.state == SDL_PRESSED) &&
               (event->key.keysym.mod & KMOD_ALT) )
     {
-        SDL_Surface *surface = SDL_GetVideoSurface();
-        if (surface != NULL)
-        {
-            Uint32 sdl_flags = surface->flags;
-            attempt_fullscreen_toggle(&surface, &sdl_flags);
-        } /* if */
+        if (SDL_WM_ToggleFullScreen(SDL_GetVideoSurface()))
+            sdl_fullscreen ^= 1;
         return(0);
     } /* if */
 
+    /* HDG: put this above the scancode lookup otherwise it is never reached */
+    if ( (event->key.keysym.sym == SDLK_PAUSE) &&
+         (event->key.state == SDL_PRESSED))
+    {
+        PausePressed = true;
+        return(0);
+    }
+
     k = handle_keypad_enter_hack(event);
     if (!k)
     {
@@ -487,14 +495,14 @@ static int sdl_key_filter(const SDL_Event *event)
         if (!k)   /* No DOS equivalent defined. */
             return(0);
     } /* if */
-
-    if (event->key.state == SDL_RELEASED)
+    
+    /* Fix elweirdo SDL capslock/numlock handling, always treat as press */
+    if ( (event->key.keysym.sym != SDLK_CAPSLOCK) &&
+         (event->key.keysym.sym != SDLK_NUMLOCK)  &&
+         (event->key.state == SDL_RELEASED) )
         k += 128;  /* +128 signifies that the key is released in DOS. */
 
-    if (event->key.keysym.sym == SDLK_PAUSE)
-        PausePressed = true;
-
-    else if (event->key.keysym.sym == SDLK_SCROLLOCK)
+    if (event->key.keysym.sym == SDLK_SCROLLOCK)
         PanicPressed = true;
 
     else
@@ -1027,7 +1035,7 @@ sdl_mouse_grabbed = 1;
   except where no such name is available.
  */
     memset(scancodes, '\0', sizeof (scancodes));
-    scancodes[SDLK_ESCAPE]          = sc_Escape;
+    scancodes[SDLK_COMMA]          = sc_Escape;
     scancodes[SDLK_1]               = sc_1;
     scancodes[SDLK_2]               = sc_2;
     scancodes[SDLK_3]               = sc_3;
@@ -1075,6 +1083,8 @@ sdl_mouse_grabbed = 1;
     scancodes[SDLK_LSHIFT]          = sc_RShift; /* sc_LShift */
     
     scancodes[SDLK_BACKSLASH]       = 0x2B;
+    /* Accept the German eszett as a backslash key */
+    scancodes[SDLK_WORLD_63]        = 0x2B;
     scancodes[SDLK_z]               = sc_Z;
     scancodes[SDLK_x]               = sc_X;
     scancodes[SDLK_c]               = sc_C;
@@ -1082,7 +1092,7 @@ sdl_mouse_grabbed = 1;
     scancodes[SDLK_b]               = sc_B;
     scancodes[SDLK_n]               = sc_N;
     scancodes[SDLK_m]               = sc_M;
-    scancodes[SDLK_COMMA]           = sc_Comma;
+    //scancodes[SDLK_COMMA]           = sc_Comma;
     scancodes[SDLK_PERIOD]          = sc_Period;
     scancodes[SDLK_SLASH]           = 0x35;
     scancodes[SDLK_RSHIFT]          = sc_RShift;
@@ -1093,6 +1103,7 @@ sdl_mouse_grabbed = 1;
     
     scancodes[SDLK_LALT]            = sc_Alt;
     scancodes[SDLK_RALT]            = sc_Alt;
+    scancodes[SDLK_MODE]            = sc_Alt;
     scancodes[SDLK_RCTRL]           = sc_Control;
     scancodes[SDLK_SPACE]           = sc_Space;
     scancodes[SDLK_CAPSLOCK]        = sc_CapsLock;
@@ -1120,7 +1131,9 @@ sdl_mouse_grabbed = 1;
     scancodes[SDLK_HOME]            = sc_Home;
     scancodes[SDLK_UP]              = sc_UpArrow;
     scancodes[SDLK_PAGEUP]          = sc_PgUp;
-    scancodes[SDLK_KP_MINUS]        = 0xE04A;
+    // Make this a normal minus, for viewport changing
+    //scancodes[SDLK_KP_MINUS]        = 0xE04A;
+    scancodes[SDLK_KP_MINUS]        = sc_Minus;
     scancodes[SDLK_KP4]             = sc_LeftArrow;
     scancodes[SDLK_KP5]             = 0x4C;
     scancodes[SDLK_KP6]             = sc_RightArrow;
