@@ -116,7 +116,9 @@ boolean dopefish;
 
 boolean newlevel = false;
 boolean infopause;
+#ifdef DOS
 boolean SOUNDSETUP=false;
+#endif
 boolean quiet = false;
 
 #if (DEVELOPMENT == 1)
@@ -153,12 +155,27 @@ void CheckCommandLineParameters( void );
 void PlayTurboGame( void );
 void Init_Tables (void);
 void CheckRemoteRidicule ( int scancode );
+void SetRottScreenRes (int Width, int Height);
 
 #ifndef DOS
 extern void crash_print (int);
 extern int setup_homedir (void);
 #endif
 
+//extern int G_argc;
+//extern char G_argv[30][80];
+int G_weaponscale;
+extern int iDropDemo;
+extern boolean iG_aimCross;
+extern boolean sdl_fullscreen;
+
+extern void ComSetTime ( void );
+extern void VH_UpdateScreen (void);
+extern void RottConsole ( void );
+extern void	ReadDelay(long delay);
+extern void RecordDemoQuery ( void );
+
+
 int main (int argc, char *argv[])
 {
     char *macwd;
@@ -232,6 +249,8 @@ int main (int argc, char *argv[])
       GetMenuInfo ();
       }
 
+   SetRottScreenRes (iGLOBAL_SCREENWIDTH, iGLOBAL_SCREENHEIGHT);
+   
 //   if (modemgame==true)
 //      {
 //      SCREENSHOTS=true;
@@ -291,6 +310,7 @@ int main (int argc, char *argv[])
             printf( "Sound FX disabled.\n" );
          }
 
+#ifdef DOS
       if ( status1 || status2 || status3 )
          {
          printf( "\n\nROTT was unable to initialize your " );
@@ -314,6 +334,7 @@ int main (int argc, char *argv[])
                  "Now entering sound setup.\n" );
          SOUNDSETUP = true;
          }
+#endif
 
       Init_Tables ();
       InitializeRNG ();
@@ -358,11 +379,13 @@ int main (int argc, char *argv[])
 //   SetBorderColor(155);
    SetViewSize(8);
 
+#ifdef DOS
    if ( SOUNDSETUP )
       {
       SwitchPalette( origpal, 35 );
       CP_SoundSetup();
       }
+#endif
 
    playstate = ex_titles;
 
@@ -433,8 +456,10 @@ void DrawRottTitle ( void )
       {
       SetTextMode();
       TurnOffTextCursor ();
+#ifdef DOS
       if (CheckParm ("SOUNDSETUP") == 0)
          {
+#endif
 #ifdef ANSIESC
          printf("\n\n\n");
 #endif
@@ -484,6 +509,7 @@ void DrawRottTitle ( void )
 #endif
 
          UL_ColorBox (0, 0, 80, 2, 0x1e);
+#ifdef DOS
          }
       else
          {
@@ -500,6 +526,7 @@ void DrawRottTitle ( void )
 
          UL_ColorBox (0, 0, 80, 1, 0x1e);
          }
+#endif
       }
    else
       {
@@ -518,7 +545,9 @@ void CheckCommandLineParameters( void )
    int i,n;
 
    infopause=false;
+#ifdef DOS
    SOUNDSETUP = false;
+#endif
    tedlevel=false;
    NoWait=false;
    NoSound=false;
@@ -555,6 +584,20 @@ void CheckCommandLineParameters( void )
       SetTextMode ();
       printf ("Rise of the Triad  (c) 1995 Apogee Software\n\n");
       printf ("COMMAND LINE PARAMETERS\n");
+      printf ("   AIM        - Give Aim Crosshair.\n");
+      printf ("   FULLSCREEN - Start in fullscreen mode\n");
+      printf ("   WINDOW     - Start in windowed mode\n");
+      printf ("   RESOLUTION - Specify the screen resolution to use\n");
+      printf ("              - next param is <widthxheight>, valid resolutions are:\n");
+      printf ("              - 320x200, 640x480 and 800x480\n");
+#if (SHAREWARE==0)
+      printf ("   FILERTL    - used to load Userlevels (RTL files)\n");
+      printf ("              - next parameter is RTL filename\n");
+      printf ("   FILERTC    - used to load Battlelevels (RTC files)\n");
+      printf ("              - next parameter is RTC filename\n");
+      printf ("   FILE       - used to load Extern WAD files\n");
+      printf ("              - next parameter is WAD filename\n");
+#endif
       printf ("   SPACEBALL  - Enable check for Spaceball.\n");
       printf ("   NOJOYS     - Disable check for joystick.\n");
       printf ("   NOMOUSE    - Disable check for mouse.\n");
@@ -566,7 +609,9 @@ void CheckCommandLineParameters( void )
       printf ("   MONO       - Enable mono-monitor support.\n");
       printf ("   SCREENSHOTS- Clean screen capture for shots.\n");
       printf ("   PAUSE      - Pauses startup screen information.\n");
+#ifdef DOS
       printf ("   SOUNDSETUP - Setup sound for ROTT\n");
+#endif
       printf ("   ENABLEVR   - Enable VR helmet input devices\n");
       printf ("   NOECHO     - Turn off sound reverb\n");
       printf ("   DEMOEXIT   - Exit program when demo is terminated\n");
@@ -577,6 +622,57 @@ void CheckCommandLineParameters( void )
       printf ("   MAXTIMELIMIT - Maximimum time to count down from\n");
       printf ("                next paramater is time in seconds\n");
       printf ("   DOPEFISH   - ?\n");
+      printf (" \n");
+      printf ("CONTROLS\n");
+      printf ("         Arrows           - Move\n");
+      printf ("         Ctrl             - Fire\n");
+      printf ("         Comma/Alt+left   - Sidestep Left\n");
+      printf ("         Period/Alt+right - Sidestep Right\n");
+      printf ("         Shift            - Run/Turn faster\n");
+      printf ("         Space            - Use/Open\n");
+      printf ("         1-4              - Choose Weapon\n");
+      printf ("         5-6              - Scale Weapon Up/Down\n");
+      printf ("         Enter            - Swap Weapon\n");
+      printf ("         Backspace        - Turn 180\n");
+      printf ("         Delete           - Drop Weapon\n");
+      printf ("         +/-              - Change Viewsize\n");
+      printf ("         PgUp/PgDn        - Look Up/Down\n");
+      printf ("         Home/End         - Aim Up/Down\n");
+      printf ("         [ ]              - Sound Volumen\n");
+      printf ("         ( )              - Music Volumen\n");
+      printf ("         Tab              - Enter Automapper\n");
+      printf (" \n");
+      printf ("AUTO-MAPPER\n");
+      printf ("         Arrows           - Scroll around\n");
+      printf ("         PgUp             - Zoom Out\n");
+      printf ("         PgDn             - Zoom In\n");
+      printf ("         Tab              - Exit Auto-Mapper\n");
+      printf (" \n");
+      printf ("HOTKEYS\n");
+      printf ("         F1               - Help\n");
+      printf ("         F2               - Save Game\n");
+      printf ("         F3               - Restore Game\n");
+      printf ("         F4               - Controls/Sound/Music\n");
+      printf ("         F5               - Change Detail Level\n");
+      printf ("         F6               - Quick Save\n");
+      printf ("         F7               - Messages On/Off\n");
+      printf ("         F8               - End Game\n");
+      printf ("         F9               - Quick Load\n");
+      printf ("         F10              - Quit\n");
+      printf ("         F11              - Gamma Correction\n");
+      printf (" \n");
+      printf ("COMM-BAT\n");
+      printf ("         F1 - F10         - RemoteRidicule(tm) sounds\n");
+      printf ("         F12              - Live RemoteRidicule\n");
+      printf ("         T                - Type message to all\n");
+      printf ("         Z                - Type directed message\n");
+      printf ("         Tab              - Toggle KillCount display\n");
+      printf (" \n");
+      printf ("SCREENSHOOT\n");
+#ifdef DOS /* makes no sense under Linux as there are no lbm viewers there */
+      printf ("         Alt+V            - Screenshoot in LBM format\n");
+#endif
+      printf ("         Alt+C            - Screenshoot in PCX format\n");
       exit (0);
       }
 
@@ -681,7 +777,9 @@ void CheckCommandLineParameters( void )
          infopause=true;
          break;
        case 13:
+#ifdef DOS
           SOUNDSETUP = true;
+#endif
           break;
        case 14:
           startlevel = (ParseNum(_argv[i + 1])-1);
@@ -722,16 +820,123 @@ void CheckCommandLineParameters( void )
 void SetupWads( void )
 {
    char  *newargs[99];
-	int argnum = 0;
-#if (SHAREWARE==0)
-   int arg;
-#endif
+   int i, arg, argnum = 0;
    char tempstr[129];
+   char *PStrings[] = {"AIM", "FULLSCREEN", "WINDOW", "RESOLUTION", NULL };
+
+   // These must be checked here so that they can override the cfg file
+   for (i = 1;i < _argc;i++)
+   {
+      arg = US_CheckParm(_argv[i],PStrings);
+      switch(arg)
+      {
+         case 0:
+            iG_aimCross = 1;
+            break;
+         case 1:
+            sdl_fullscreen = 1;
+            break;
+         case 2:
+            sdl_fullscreen = 0;
+            break;
+         case 3:
+            i++;
+            if (i < _argc)
+            {
+               int width, height;
+               if ( (sscanf(_argv[i], "%dx%d", &width, &height) == 2) &&
+                    ( ( (width == 320) && (height == 200) ) ||
+                      ( (width == 640) && (height == 480) ) ||
+                      ( (width == 800) && (height == 480) ) ) )
+               {
+                 iGLOBAL_SCREENWIDTH  = width;
+                 iGLOBAL_SCREENHEIGHT = height;
+               }
+               else
+                  printf("Invalid resolution parameter: %s\n", _argv[i]);
+            }
+            else
+               printf("Missing resolution parameter\n");
+            break;
+      }
+   }
+
 
 #if (SHAREWARE==0)
+   // Check for rtl files 	
+   arg = CheckParm ("filertl");
+   if (arg!=0)
+   {
+	   FILE *f;char buf[32];
+	   if (_argv[arg+1] != 0) { //are there a filename included
+		   strcpy (tempstr,_argv[arg+1]);//copy it to tempstr
+		   if (strlen (tempstr) < MAX_PATH) {
+			   if (access (tempstr, 0) != 0) { //try open
+				  	strcat (tempstr,".rtc");//non exists, try add .rtc
+					if (access (tempstr, 0) != 0) { //try open again
+						//stil no useful filename
+						strcat (tempstr," not found, skipping RTL file ");
+						printf(tempstr);
+						goto NoRTL;
+					}
+			   }
+			   if((f = fopen( tempstr, "r" )) == NULL ){ //try opnong file
+					strcat (tempstr," not could not be opened, skipping RTL file ");
+					printf(tempstr);
+					goto NoRTL;
+			   }else{
+					fread(buf,3,3,f);//is the 3 first letters RTL (RTC)
+				    if (((strstr(buf,"RTL") != 0)||strstr(buf,"RTC") != 0)) {
+						strcpy (GameLevels.file,tempstr);
+						GameLevels.avail++;
+						strcpy (buf,"Adding ");
+						strcat (buf,tempstr);
+						printf(buf);
+					}
+					fclose(f);
+			   }
+		   }
+	   }else{printf("Missing RTL filename");}
+   }
+NoRTL:;
+   // Check for rtc files
+   arg = CheckParm ("filertc");
+   if (arg!=0)
+   {
+	   FILE *f;char buf[32];
+	   if (_argv[arg+1] != 0) { //are there a filename included
+		   strcpy (tempstr,_argv[arg+1]);//copy it to tempstr
+		   if (strlen (tempstr) < MAX_PATH) {
+			   if (access (tempstr, 0) != 0) { //try open
+				  	strcat (tempstr,".rtc");//non exists, try add .rtc
+					if (access (tempstr, 0) != 0) { //try open again
+						//stil no useful filename
+						strcat (tempstr," not found, skipping RTC file ");
+						printf(tempstr);
+						goto NoRTL;
+					}
+			   }
+			   if((f = fopen( tempstr, "r" )) == NULL ){ //try opening file
+					strcat (tempstr," not could not be opened, skipping RTC file ");
+					printf(tempstr);
+					goto NoRTL;
+			   }else{
+					fread(buf,3,3,f);//is the 3 first letters RTL (RTC)
+				    if (((strstr(buf,"RTL") != 0)||strstr(buf,"RTC") != 0)) {
+						strcpy (BattleLevels.file,tempstr);
+						BattleLevels.avail++;
+						strcpy (buf,"Adding ");
+						strcat (buf,tempstr);
+						printf(buf);
+					}
+					fclose(f);
+			   }
+		   }
+	   }else{printf("Missing RTC filename");}
+   }
+NoRTC:;
 
    // Check for User wads
-
    arg = CheckParm ("file");
    if (arg!=0)
       {
@@ -763,9 +968,9 @@ void SetupWads( void )
    // Normal ROTT wads
 
 #if (SHAREWARE)
-   newargs [argnum++] = "huntbgin.wad";
+   newargs [argnum++] = DATADIR "HUNTBGIN.WAD";
 #else
-   newargs [argnum++] = "darkwar.wad";
+   newargs [argnum++] = DATADIR "DARKWAR.WAD";
 #endif
 
 //   newargs [argnum++] = "credits.wad";
@@ -785,7 +990,7 @@ void SetupWads( void )
       }
    else
       {
-      newargs [argnum++] = "remote1.rts";
+      newargs [argnum++] = DATADIR "REMOTE1.RTS";
       }
 
    newargs [argnum++] = NULL;
@@ -836,7 +1041,7 @@ void Init_Tables (void)
 	blockstart = &blockstarts[0];
 	for (y=0;y<UPDATEHIGH;y++)
 		for (x=0;x<UPDATEWIDE;x++)
-			*blockstart++ = SCREENWIDTH*16*y+x*TILEWIDTH;
+			*blockstart++ = iG_SCREENWIDTH*16*y+x*TILEWIDTH;
 
 	for (i = 0; i < 0x300; i++)
 		*(origpal+(unsigned int)i) = (*(origpal+(unsigned int)i))>>2;
@@ -940,14 +1145,17 @@ void GameLoop (void)
          CalcTics();
          CalcTics();
 
+
          playstate = ex_titles;
          }
 
 		switch (playstate)
 		   {
          case ex_titles:
+		 
             BATTLE_Shutdown();
             MU_StartSong(song_title);
+			EnableScreenStretch();
             if ((NoWait==false)&&(!modemgame))
                {
                byte dimpal[768];
@@ -962,7 +1170,7 @@ void GameLoop (void)
                while ((!LastScan) && (!IN_GetMouseButtons()))
                   {
                   int i;
-                  unsigned tempbuf;
+                  byte *tempbuf;
                   MenuFadeOut();
                   ClearGraphicsScreen();
                   SetPalette(&dimpal[0]);
@@ -983,8 +1191,9 @@ void GameLoop (void)
                   I_Delay(30);
                   SD_Play (SD_ACTORSQUISHSND);
                   tempbuf=bufferofs;
-                  bufferofs=displayofs;
+                  bufferofs=page1start; // fixed, was displayofs
                   DrawNormalSprite(320-94,200-41,W_GetNumForName("rsac"));
+						VW_UpdateScreen(); // fixed, was missing
                   bufferofs=tempbuf;
                   I_Delay(30);
 
@@ -993,7 +1202,7 @@ void GameLoop (void)
                      break;
                      }
 
-                  DoCreditScreen ();
+					DoCreditScreen ();
                   if ((!LastScan) && (!IN_GetMouseButtons()))
                      CheckHighScore (0, 0, false);
 #if (SHAREWARE==0)
@@ -1036,10 +1245,14 @@ void GameLoop (void)
                NoWait = false;
                SwitchPalette(origpal,35);
                CP_MainMenu();
+
                }
          break;
 
          case ex_resetgame:
+
+  // SetTextMode (  ); //12345678
+	    EnableScreenStretch();//bna++ shut on streech mode 
             InitCharacter();
 
             InitializeMessages();
@@ -1123,6 +1336,9 @@ void GameLoop (void)
 
             MenuFixup ();
             playstate=ex_stillplaying;
+
+	    DisableScreenStretch();//bna++ shut off streech mode
+
          break;
 
          case ex_stillplaying:
@@ -1143,10 +1359,10 @@ void GameLoop (void)
 
          case ex_died:
             loadit = done = false;
-
+//		   SetTextMode (  ); //12345678
             Died ();
             StopWind();
-
+			 DisableScreenStretch();//bna++ shut off streech mode
             while (damagecount>0)
                DoBorderShifts();
 
@@ -1291,8 +1507,7 @@ void GameLoop (void)
 
                WaitKeyUp();
                LastScan = 0;
-               while (!LastScan)
-                  ;
+               while (!LastScan) IN_UpdateKeyboard();	// Thanks again DrLex
                LastScan=0;
                }
 #endif
@@ -1328,6 +1543,11 @@ void GameLoop (void)
             demoplayback = false;
 
             Z_FreeTags (PU_LEVELSTRUCT, PU_LEVELEND);       // Free current level
+            if (predemo_violence != -1)
+            {
+               gamestate.violence = predemo_violence;
+               predemo_violence = -1;
+            }
             playstate=ex_titles;
          break;
 
@@ -1372,12 +1592,12 @@ void GameLoop (void)
    waminot();
    }
 
-boolean CheckForQuickLoad
-   (
-   void
-   )
+boolean CheckForQuickLoad  (void )
 
    {
+
+   EnableScreenStretch();//bna++
+
    if ( pickquick )
       {
       SetupMenuBuf();
@@ -1405,7 +1625,11 @@ boolean CheckForQuickLoad
 
 void ShutDown ( void )
 {
-   if ( ( standalone == false ) || ( SOUNDSETUP ) )
+   if ( ( standalone == false )
+#ifdef DOS
+        || ( SOUNDSETUP ) 
+#endif
+   )
       {
       WriteConfig ();
       }
@@ -1497,8 +1721,10 @@ void QuitGame ( void )
    printf("LIGHTRATE =%ld\n",GetLightRateTile());
    printf("\nCENTERY=%ld\n",centery);
 #else
+#ifdef DOS
    if ( !SOUNDSETUP )
       {
+#endif
 #if (SHAREWARE==0)
       txtscn = (byte *) W_CacheLumpNum (W_GetNumForName ("regend"), PU_CACHE, CvtNull, 1);
 #else
@@ -1525,15 +1751,19 @@ void QuitGame ( void )
 
       UL_printf (itoa(ROTTMINORVERSION,&buf[0],10));
 #endif
+#ifdef DOS
       }
 #endif
+#endif
 
+#ifdef DOS
    if ( SOUNDSETUP )
       {
       printf( "\nSound setup complete.\n"
               "Type ROTT to run the game.\n" );
       }
    ShutDown();
+#endif
 
    exit(0);
 }
@@ -1716,6 +1946,17 @@ void PauseLoop ( void )
       oldpolltime++;
       if (GamePaused==false)
          {
+   			//bna++ section
+		  if (( playstate == ex_stillplaying )&&(iGLOBAL_SCREENWIDTH > 320)){
+				pic_t *shape;
+				shape =  ( pic_t * )W_CacheLumpName( "backtile", PU_CACHE, Cvt_pic_t, 1 );
+				DrawTiledRegion( 0, 16, iGLOBAL_SCREENWIDTH, iGLOBAL_SCREENHEIGHT - 32, 0, 16, shape );
+				DisableScreenStretch();//dont strech when we go BACK TO GAME
+				DrawPlayScreen(true);//repaint ammo and life stat
+				VW_UpdateScreen ();//update screen
+		  }
+		  StartupClientControls();
+		   //bna section end
          break;
          }
 		}
@@ -2198,6 +2439,7 @@ void PollKeyboard
       // Shrink screen
       if ( Keyboard[ sc_Minus ] )
          {
+         Keyboard[ sc_Minus ] = false; // HDG debounce
          if ( viewsize > 0 )
             {
             viewsize--;
@@ -2208,6 +2450,7 @@ void PollKeyboard
       // Expand screen
       if ( Keyboard[ sc_Plus ] )
          {
+         Keyboard[ sc_Plus ] = false; // HDG debounce
          if ( viewsize < MAXVIEWSIZES - 1 )
             {
             viewsize++;
@@ -2421,12 +2664,29 @@ void PollKeyboard
             {
             SaveScreen( false );
             }
+#ifdef DOS /* makes no sense under Linux as there are no lbm viewers there */
          else if ( Keyboard[ sc_Alt] && Keyboard[ sc_V ] )
             {
             SaveScreen( true );
             }
+#endif
       #endif
       }
+#ifdef USE_SDL
+      /* SDL doesn't send proper release events for these */
+      if (Keystate[sc_CapsLock])
+      {
+         Keystate[sc_CapsLock]++;
+         if (Keystate[sc_CapsLock] == 3)
+            Keystate[sc_CapsLock] = 0;
+      }
+      if (Keystate[0x45]) /* numlock */ 
+      {
+         Keystate[0x45]++;
+         if (Keystate[0x45] == 3)
+            Keystate[0x45] = 0;
+      }
+#endif
    waminot();
    }
 
@@ -2708,7 +2968,7 @@ void WriteLBMfile (char *filename, byte *data, int width, int height)
    int     handle;
    int     i;
 
-   lbm = lbmptr = (byte *) SafeMalloc (65000);
+   lbm = lbmptr = (byte *) SafeMalloc ((iGLOBAL_SCREENWIDTH*iGLOBAL_SCREENHEIGHT)+4000);
 
 //
 // start FORM
@@ -2917,7 +3177,8 @@ void SaveScreen (boolean saveLBM)
       ThreeDRefresh ();
    doublestep = 2 - DetailLevel;
 
-   buffer = (byte *) SafeMalloc (65000);
+   //buffer = (byte *) SafeMalloc (65000);
+   buffer = (byte *) SafeMalloc ((iGLOBAL_SCREENHEIGHT*iGLOBAL_SCREENWIDTH)+4000);
 
 #if (BETA == 1)
    if (SCREENSHOTS == false)
@@ -2950,21 +3211,32 @@ void SaveScreen (boolean saveLBM)
    }
 #endif
 
-   VL_CopyPlanarPageToMemory(screen,buffer);
+
 
    GetFileName (saveLBM);
    GetPathFromEnvironment( filename, ApogeePath, savename );
+   //   
+	memcpy(buffer,screen , iGLOBAL_SCREENWIDTH*iGLOBAL_SCREENHEIGHT);//bna	
+   //bna--VL_CopyPlanarPageToMemory(screen,buffer);
 
    if (saveLBM)
    {
-      WriteLBMfile (filename, buffer, 320, 200);
+      WriteLBMfile (filename, buffer, iGLOBAL_SCREENWIDTH, iGLOBAL_SCREENHEIGHT);
+#if (DEVELOPMENT == 1)
       while (Keyboard[sc_CapsLock] && Keyboard[sc_C])
+#else
+      while (Keyboard[sc_Alt] && Keyboard[sc_V])
+#endif
            IN_UpdateKeyboard ();
    }
    else
    {
       WritePCX (filename, buffer);
+#if (DEVELOPMENT == 1)
       while (Keyboard[sc_CapsLock] && Keyboard[sc_X])
+#else
+      while (Keyboard[sc_Alt] && Keyboard[sc_C])
+#endif
            IN_UpdateKeyboard ();
    }
 
@@ -2996,7 +3268,7 @@ void WritePCX (char * file, byte * source)
        *    unlike in CONVERT.EXE.
        */
 
-   memset (&pcxHDR, sizeof(PCX_HEADER), 0);
+   memset (&pcxHDR, 0, sizeof(PCX_HEADER));
 
    pcxHDR.manufacturer  = 10;
    pcxHDR.version       = 5;
@@ -3004,15 +3276,15 @@ void WritePCX (char * file, byte * source)
 
    pcxHDR.bitsperpixel  = 8;           //bpp;
    pcxHDR.xmin          = pcxHDR.ymin = 0;
-   pcxHDR.xmax          = 319;         //bitmap->box.w - 1;
-   pcxHDR.ymax          = 199;         //bitmap->box.h - 1;
-   pcxHDR.hres          = 320;         //N_COLUMNS;
-   pcxHDR.vres          = 200;         //N_LINES;
+   pcxHDR.xmax          = iGLOBAL_SCREENWIDTH - 1;
+   pcxHDR.ymax          = iGLOBAL_SCREENHEIGHT - 1;
+   pcxHDR.hres          = iGLOBAL_SCREENWIDTH;         //N_COLUMNS;
+   pcxHDR.vres          = iGLOBAL_SCREENHEIGHT;         //N_LINES;
 
   // bytesperline doesn't take into account multiple planes.
   // Output in same format as bitmap (planar vs packed).
   //
-   pcxHDR.bytesperline  = 320;         //bitmap->width;
+   pcxHDR.bytesperline  = iGLOBAL_SCREENWIDTH;         //bitmap->width;
 
    pcxHDR.nplanes       = 1;           //bitmap->planes;
    pcxHDR.reserved      = 0;
@@ -3029,19 +3301,19 @@ void WritePCX (char * file, byte * source)
   //
    SafeWrite(pcxhandle,&pcxHDR, sizeof (PCX_HEADER));
 
-   memset (buffer1, GAP_SIZE, 0);
+   memset (buffer1, 0, GAP_SIZE);
 
    SafeWrite (pcxhandle, &buffer1, GAP_SIZE);
 
-   tempbuffer = (byte *) SafeMalloc (65000);
+   tempbuffer = (byte *) SafeMalloc ((iGLOBAL_SCREENHEIGHT*iGLOBAL_SCREENWIDTH)+4000);
    bptr = tempbuffer;
    totalbytes = 0;
 
   //
   // Write to a bit-packed file.
   //
-	for (y = 0;  y < 200;  ++y) 		// for each line in band
-		if (PutBytes (((unsigned char *) (source+(y*320))),
+	for (y = 0;  y < iGLOBAL_SCREENHEIGHT;  ++y) 		// for each line in band
+		if (PutBytes (((unsigned char *) (source+(y*iGLOBAL_SCREENWIDTH))),
 						  pcxHDR.bytesperline))
          Error ("Error writing PCX bit-packed line!\n");
 
