@@ -333,7 +333,7 @@ void Error (char *error, ...)
 
    inerror++;
    if (inerror > 1)
-      return;
+      abort();
 
 
 	SetTextMode ();
@@ -435,13 +435,14 @@ void Error (char *error, ...)
       }
 
    close(handle);
-#endif
 
    if ( SOUNDSETUP )
       {
       getch();
       }
 
+#endif
+
    #if USE_SDL
    SDL_Quit();
    #endif
@@ -927,7 +928,7 @@ int _dos_findfirst(char *filename, int x, struct find_t *f)
     {
         *ptr = '\0';
         f->dir = opendir(f->pattern);
-        memmove(f->pattern, ptr + 1, strlen(ptr) + 1);
+        memmove(f->pattern, ptr + 1, strlen(ptr + 1) + 1);
     }
 
     return(_dos_findnext(f));
@@ -1171,7 +1172,7 @@ short	KeepShort (short l)
 }
 
 
-long	SwapLong (long l)
+int	SwapLong (int l)
 {
 	byte	b1,b2,b3,b4;
 
@@ -1180,10 +1181,10 @@ long	SwapLong (long l)
 	b3 = (l>>16)&255;
 	b4 = (l>>24)&255;
 
-	return ((long)b1<<24) + ((long)b2<<16) + ((long)b3<<8) + b4;
+	return ((int)b1<<24) + ((int)b2<<16) + ((int)b3<<8) + b4;
 }
 
-long	KeepLong (long l)
+int	KeepLong (int l)
 {
 	return l;
 }
@@ -1194,7 +1195,7 @@ long	KeepLong (long l)
 #undef SwapShort
 #undef SwapLong
 
-void SwapIntelLong(long *l)
+void SwapIntelLong(int *l)
 {
     *l = IntelLong(*l);
 }
@@ -1204,7 +1205,7 @@ void SwapIntelShort(short *s)
     *s = IntelShort(*s);
 }
 
-void SwapIntelLongArray(long *l, int num)
+void SwapIntelLongArray(int *l, int num)
 {
     while (num--) {
         SwapIntelLong(l);
@@ -1850,9 +1851,15 @@ boolean UL_ChangeDirectory (char *path)
 
    return (true);
 #else
-	STUB_FUNCTION;
+	if (!*path) {
+		return true;
+	}
 	
-	return false;
+	if (chdir (path) == -1) {
+	         return (false);
+	}
+	
+	return true;
 #endif
 }
 
